@@ -5,7 +5,7 @@
 
     Configuration introspection and defaults.
 
-    :copyright: (c) 2009 - 2011 by Ask Solem.
+    :copyright: (c) 2009 - 2012 by Ask Solem.
     :license: BSD, see LICENSE for more details.
 
 """
@@ -16,6 +16,8 @@ import sys
 from collections import deque
 from datetime import timedelta
 
+from ..utils.functional import memoize
+
 is_jython = sys.platform.startswith("java")
 is_pypy = hasattr(sys, "pypy_version_info")
 
@@ -73,13 +75,21 @@ NAMESPACES = {
         "CONNECTION_TIMEOUT": Option(4, type="int"),
         "CONNECTION_RETRY": Option(True, type="bool"),
         "CONNECTION_MAX_RETRIES": Option(100, type="int"),
-        "POOL_LIMIT": Option(None, type="int"),
+        "POOL_LIMIT": Option(10, type="int"),
         "INSIST": Option(False, type="bool",
                          deprecate_by="2.4", remove_by="3.0"),
         "USE_SSL": Option(False, type="bool"),
         "TRANSPORT": Option(None, type="string"),
         "TRANSPORT_OPTIONS": Option({}, type="dict"),
     },
+    "CASSANDRA": {
+        "COLUMN_FAMILY": Option(None, type="string"),
+        "DETAILED_MODE": Option(False, type="bool"),
+        "KEYSPACE": Option(None, type="string"),
+        "READ_CONSISTENCY": Option(None, type="string"),
+        "SERVERS": Option(None, type="list"),
+        "WRITE_CONSISTENCY": Option(None, type="string"),
+    },
     "CELERY": {
         "ACKS_LATE": Option(False, type="bool"),
         "ALWAYS_EAGER": Option(False, type="bool"),
@@ -88,6 +98,7 @@ NAMESPACES = {
                 alt="CELERY_TASK_RESULT_EXPIRES"),
         "AMQP_TASK_RESULT_CONNECTION_MAX": Option(1, type="int",
             remove_by="2.5", alt="BROKER_POOL_LIMIT"),
+        "ANNOTATIONS": Option(type="any"),
         "BROADCAST_QUEUE": Option("celeryctl"),
         "BROADCAST_EXCHANGE": Option("celeryctl"),
         "BROADCAST_EXCHANGE_TYPE": Option("fanout"),
@@ -102,6 +113,7 @@ NAMESPACES = {
         "DEFAULT_EXCHANGE_TYPE": Option("direct"),
         "DEFAULT_DELIVERY_MODE": Option(2, type="string"),
         "EAGER_PROPAGATES_EXCEPTIONS": Option(False, type="bool"),
+        "ENABLE_UTC": Option(False, type="bool"),
         "EVENT_SERIALIZER": Option("json"),
         "IMPORTS": Option((), type="tuple"),
         "IGNORE_RESULT": Option(False, type="bool"),
@@ -112,6 +124,7 @@ NAMESPACES = {
         "REDIS_PORT": Option(None, type="int"),
         "REDIS_DB": Option(None, type="int"),
         "REDIS_PASSWORD": Option(None, type="string"),
+        "REDIS_MAX_CONNECTIONS": Option(None, type="int"),
         "RESULT_BACKEND": Option(None, type="string"),
         "RESULT_DB_SHORT_LIVED_SESSIONS": Option(False, type="bool"),
         "RESULT_DBURI": Option(),
@@ -135,16 +148,23 @@ NAMESPACES = {
                 "interval_step": 0.2}, type="dict"),
         "TASK_RESULT_EXPIRES": Option(timedelta(days=1), type="int"),
         "TASK_SERIALIZER": Option("pickle"),
+        "TIMEZONE": Option(None, type="string"),
         "TRACK_STARTED": Option(False, type="bool"),
         "REDIRECT_STDOUTS": Option(True, type="bool"),
         "REDIRECT_STDOUTS_LEVEL": Option("WARNING"),
         "QUEUES": Option(None, type="dict"),
+        "SECURITY_KEY": Option(None, type="string"),
+        "SECURITY_CERTIFICATE": Option(None, type="string"),
+        "SECURITY_CERT_STORE": Option(None, type="string"),
     },
     "CELERYD": {
         "AUTOSCALER": Option("celery.worker.autoscale.Autoscaler"),
+        "AUTORELOADER": Option("celery.worker.autoreload.Autoreloader"),
+        "BOOT_STEPS": Option((), type="tuple"),
         "CONCURRENCY": Option(0, type="int"),
-        "ETA_SCHEDULER": Option(None, type="str"),
+        "ETA_SCHEDULER": Option(None, type="string"),
         "ETA_SCHEDULER_PRECISION": Option(1.0, type="float"),
+        "FORCE_EXECV": Option(False, type="bool"),
         "HIJACK_ROOT_LOGGER": Option(True, type="bool"),
         "CONSUMER": Option("celery.worker.consumer.Consumer"),
         "LOG_FORMAT": Option(DEFAULT_PROCESS_LOG_FMT),
@@ -202,6 +222,7 @@ def flatten(d, ns=""):
                 stack.append((name + key + '_', value))
             else:
                 yield name + key, value
+DEFAULTS = dict((key, value.default) for key, value in flatten(NAMESPACES))
 
 
 def find_deprecated_settings(source):
@@ -214,4 +235,24 @@ def find_deprecated_settings(source):
                             alternative=opt.alt)
 
 
-DEFAULTS = dict((key, value.default) for key, value in flatten(NAMESPACES))
+@memoize(maxsize=None)
+def find(name, namespace="celery"):
+    # - Try specified namespace first.
+    namespace = namespace.upper()
+    for key, value in NAMESPACES[namespace].iteritems():
+        if key.lower() == name.lower():
+            return namespace, key, value
+
+    # - Try all the other namespaces.
+    for ns, keys in NAMESPACES.iteritems():
+        if isinstance(keys, dict):
+            if ns != namespace:
+                for key, value in keys.iteritems():
+                    if key.lower() == name.lower():
+                        return ns, key, value
+        else:
+            if ns.lower() == name.lower():
+                return None, key, value
+
+    # - See if name is a qualname last.
+    return None, name.upper(), DEFAULTS[name.upper()]
