@@ -2,15 +2,18 @@
 from __future__ import absolute_import
 from __future__ import with_statement
 
+if __name__ == "__main__" and __package__ is None:
+    __package__ = "celery.bin.celeryd_detach"
+
 import os
 import sys
 
 from optparse import OptionParser, BadOptionError
 
-from celery import __version__
-from celery.platforms import detached
+from .. import __version__
+from ..platforms import detached
 
-from celery.bin.base import daemon_options
+from .base import daemon_options
 
 OPTION_LIST = daemon_options(default_pidfile="celeryd.pid")
 
