@@ -88,6 +88,9 @@ Examples
 """
 from __future__ import absolute_import
 
+if __name__ == "__main__" and __package__ is None:
+    __package__ = "celery.bin.celeryd_multi"
+
 import errno
 import os
 import signal
@@ -98,10 +101,10 @@ from collections import defaultdict
 from subprocess import Popen
 from time import sleep
 
-from celery import __version__
-from celery.platforms import shellsplit
-from celery.utils import term
-from celery.utils.encoding import from_utf8
+from .. import __version__
+from ..platforms import shellsplit
+from ..utils import term
+from ..utils.encoding import from_utf8
 
 SIGNAMES = set(sig for sig in dir(signal)
                         if sig.startswith("SIG") and "_" not in sig)
@@ -290,7 +293,7 @@ class MultiTool(object):
             self.note("")
 
     def getpids(self, p, cmd, callback=None):
-        from celery import platforms
+        from .. import platforms
         pidfile_template = p.options.setdefault("--pidfile", "celeryd@%n.pid")
 
         nodes = []
