@@ -2,15 +2,18 @@
 from __future__ import absolute_import
 from __future__ import with_statement
 
+if __name__ == "__main__" and __package__ is None:
+    __package__ = "celery.bin.celeryev"
+
 import os
 import sys
 
 from functools import partial
 
-from celery import platforms
-from celery.platforms import detached
+from .. import platforms
+from ..platforms import detached
 
-from celery.bin.base import Command, Option, daemon_options
+from .base import Command, Option, daemon_options
 
 
 class EvCommand(Command):
@@ -41,19 +44,19 @@ class EvCommand(Command):
             os.chdir(workdir)
 
     def run_evdump(self):
-        from celery.events.dumper import evdump
+        from ..events.dumper import evdump
         self.set_process_status("dump")
         return evdump(app=self.app)
 
     def run_evtop(self):
-        from celery.events.cursesmon import evtop
+        from ..events.cursesmon import evtop
         self.set_process_status("top")
         return evtop(app=self.app)
 
     def run_evcam(self, camera, logfile=None, pidfile=None, uid=None,
             gid=None, umask=None, working_directory=None,
             detach=False, **kwargs):
-        from celery.events.snapshot import evcam
+        from ..events.snapshot import evcam
         workdir = working_directory
         self.set_process_status("cam")
         kwargs["app"] = self.app
