@@ -5,15 +5,24 @@
 
     This module contains various utilities relating to dates and times.
 
-    :copyright: (c) 2009 - 2011 by Ask Solem.
+    :copyright: (c) 2009 - 2012 by Ask Solem.
     :license: BSD, see LICENSE for more details.
 
 """
 from __future__ import absolute_import
 
+from kombu.utils import cached_property
+
 from datetime import datetime, timedelta
+from dateutil import tz
 from dateutil.parser import parse as parse_iso8601
 
+try:
+    import pytz
+except ImportError:
+    pytz = None  # noqa
+
+
 DAYNAMES = "sun", "mon", "tue", "wed", "thu", "fri", "sat"
 WEEKDAYS = dict((name, dow) for name, dow in zip(DAYNAMES, range(7)))
 
@@ -21,6 +30,7 @@ RATE_MODIFIER_MAP = {"s": lambda n: n,
                      "m": lambda n: n / 60.0,
                      "h": lambda n: n / 60.0 / 60.0}
 
+
 HAVE_TIMEDELTA_TOTAL_SECONDS = hasattr(timedelta, "total_seconds")
 
 TIME_UNITS = (("day", 60 * 60 * 24.0, lambda n: "%.2f" % n),
@@ -29,6 +39,47 @@ TIME_UNITS = (("day", 60 * 60 * 24.0, lambda n: "%.2f" % n),
               ("second", 1.0, lambda n: "%.2f" % n))
 
 
+class UnknownTimezone(Exception):
+    """No specification exists for the timezone specified.  Consider
+    installing the pytz library to get access to more timezones."""
+
+
+def _is_naive(dt):
+    return bool(dt.tzinfo)
+
+
+class _Zone(object):
+
+    def tz_or_local(self, tzinfo=None):
+        if tzinfo is None:
+            return self.local
+        return self.get_timezone(tzinfo)
+
+    def to_local(self, dt, local=None, orig=None):
+        return dt.replace(tzinfo=orig or self.utc).astimezone(
+                    self.tz_or_local(local))
+
+    def get_timezone(self, zone):
+        if isinstance(zone, basestring):
+            if pytz:
+                return pytz.timezone(zone)
+            zone = tz.gettz(zone)
+            if zone is None:
+                raise UnknownTimezone(UnknownTimezone.__doc__)
+            return zone
+        return zone
+
+    @cached_property
+    def local(self):
+        return tz.tzlocal()
+
+    @cached_property
+    def utc(self):
+        return self.get_timezone("UTC")
+
+timezone = _Zone()
+
+
 def maybe_timedelta(delta):
     """Coerces integer to timedelta if `delta` is an integer."""
     if isinstance(delta, (int, float)):
@@ -45,6 +96,7 @@ if HAVE_TIMEDELTA_TOTAL_SECONDS:   # pragma: no cover
 
         """
         return max(delta.total_seconds(), 0)
+
 else:  # pragma: no cover
 
     def timedelta_seconds(delta):  # noqa
@@ -91,10 +143,10 @@ def remaining(start, ends_in, now=None, relative=False):
         calculated using :func:`delta_resolution` (i.e. rounded to the
         resolution of `ends_in`).
     :keyword now: Function returning the current time and date,
-        defaults to :func:`datetime.now`.
+        defaults to :func:`datetime.utcnow`.
 
     """
-    now = now or datetime.now()
+    now = now or datetime.utcnow()
 
     end_date = start + ends_in
     if relative:
